%plots the data corresponding to all the sensors specified in the
%sensor_list parameter for the specified day, month, and year.
%parameters:
%list of sensors
%
%day: is the day to use in the plot
%month: is the month to use in the plot
%year: is the year to use in the plot
%sensor_list: is a vector containing the list of the sensor numbers to plot.
%if this parameter is [], the data of all the sensors will be plotted.
%time_interval: is the time interval to plot in seconds
%sensor_data: is the matrix containing all the sensor data
%locations: matrix containing all the sensor locations
%types: matrix containing all the sensor types
%w_interval: interval to plot before and after each activity time
%
%usage example:
%
%ploting all the sensors in dir 'd:\data\'
%graphSensorDayLocations(27,03,2003,[],[],sensor_data,locations,types,4,0);
%
%ploting only sensors 51,54,61,
%graphSensorDayLocations(27,03,2003,[51 54 61],[],sensor_data,locations,types,4,0);
function graphSensorDayLocations(day,month,year,sensor_list,time_interval,sensors_data,locations,types,level,w_interval);

%constants---------
BLACK = false;    %produce a plot in color black
%------------------

figure;

%finding all the sensors if necessary
if(isempty(sensor_list)),
   %calculating all the different sensors in the dataset
   sensor_list = getSensorList(sensors_data);
end
sensor_list;

%specifying the time intervals if necessary
if(isempty(time_interval))
    time_interval(1) = 1;
    time_interval(2) = 24*60*60;
end

%getting the indexes of all the different rooms
diff_rooms = getRoomList(sensors_data);

%calculating the labels of the rooms
room_labels = locations(diff_rooms);

%calculating day of year
ydays = yearday(day,month,year);

%sorting the sensors accordingly to location
location_list = [];
for i=1:length(diff_rooms),
    %finding all sensors activated in current room
    index = find(sensors_data(:,1)==ydays & sensors_data(:,7)==diff_rooms(i));
    location_list = [location_list;sensors_data(index,6)];
end


%eliminating repeated sensors
diff_sensors_loc = [];
pos = 1;
for i=1:size(location_list,1),
   if(isempty(find(location_list(diff_sensors_loc)==location_list(i))))
       diff_sensors_loc(pos) = i;
       pos = pos+1;
   end
end 

sensor_list = location_list(diff_sensors_loc);


%calculating day of week
dey = weekday(datenum(year,month, day));
weekdays = {'Sunday' 'Monday' 'Tuesday' 'Wednesday' 'Thursday' 'Friday' 'Saturday'};
weekdays{dey}

%calculating the offset for each sensor plot
offset = length(sensor_list)*2;

%setting axis of the plot
hold on;
%axis([(activities_data(activities_day_index(1),3)-w_interval) (activities_data(activities_day_index(1),6)+w_interval) 0 offset+1]);

xmin = time_interval(1)-w_interval;
xmax = time_interval(2)+w_interval;
ymin = 0;
ymax = offset+1;

axis([xmin xmax ymin ymax]);

%changing axis properties
set(gca,'FontSize',8);
set(gca,'TickDir','out');
zoom off;

%plotting vertical lines in activity limits
liney = 0:0.1:offset+1;
linex = ones(length(liney),1)*time_interval(1);
plot(linex,liney,'r');
linex = ones(length(liney),1)*time_interval(2);
plot(linex,liney,'r');

%drawing date text
sinfo = sprintf('%s, %d/%d/%d',weekdays{dey},month,day,year);
h = text(-1000,-1000,sinfo);
z = get(h);
height = z.Extent(4);
text( time_interval(1)-w_interval +((time_interval(2)+w_interval)-(time_interval(1)-w_interval))/4,offset+height,sinfo);

%putting the label of the sensor number
fticks = 1:2:offset;
set(gca,['y','tick'],fticks);
%fticks = get(gca,['y','tick'])
labels = {};  %creating the labels for the axis
row = 1;
for i=(length(fticks)):-1:1,
    
    %finding the location and type information
    idx = find(sensors_data(:,6)==sensor_list(i));
    
    if(level==1),
        sinfo = sprintf('%d',sensor_list(i));                              
    elseif (level==2),
        sinfo = sprintf('%d',sensor_list(i));                                  
    elseif (level==3),
        sinfo = sprintf('%d %s',sensor_list(i),locations{sensors_data(idx(1),7)});                                  
    else
        sinfo = sprintf('%d %s %s',sensor_list(i),locations{sensors_data(idx(1),7)},types{sensors_data(idx(1),8)});                 
    end
    labels{row,1} = sinfo;
    row = row + 1;      
end
set(gca,['y','ticklabel'],labels); %setting the tick labels
%end labels


%for all the sensors in the list, plot signal
last_location = [];
map = colormap('prism');
k=1;
col = map(k,:);
for j=1:length(sensor_list)
    %creating vector for containing all seconds in a day
    tday = zeros(24*60*60,1);

    %finding the sensor data for the specified day and sensor
    index = find(sensors_data(:,1)==ydays & sensors_data(:,6)==sensor_list(j)  ...
        & sensors_data(:,3)<(time_interval(2)+w_interval) & sensors_data(:,3)>(time_interval(1)-w_interval));

    %filling vector with ones between sensor activations
    %tday(sensors_data(index,3))=1;
    for i=1:length(index),
        if(level==1)
           tday(sensors_data(index(i),3))=1;
           tday(sensors_data(index(i),4))=1;
        else
           tday(sensors_data(index(i),3):sensors_data(index(i),4))=1;  
        end
    end    

    %adding an offset to the
    offset = offset - 1;
    tday = tday + offset;
    offset = offset - 1;
    
    %changing color
    if(last_location~=sensors_data(index(i),7))  
        k = k+1;
       if(k>=length(map)) 
           k = 1; 
       end
       col = map(k,:);
        if(k==3 || k ==9)
           col = [0 0 0];
       end    
    end
 
    if(BLACK)
        col = [0 0 0];
    end
    
    %ploting the activation time of all sensors
    h = plot(tday);
    set(h,'Color',col);
    
    last_location = sensors_data(index(i),7);
end
colormap('default');

%changing the x-axis scale text
[xlabels,interval] = timeTicks(xmin,xmax,20,0,0,1);
set(gca,'XTick',xmin:interval:xmax,'XTickLabel',xlabels);

%creating the name of the file containing manually
%anotated activities
if(day<10)
    day = sprintf('0%d',day);
else
    day = sprintf('%d',day);
end
if(month<10)
    month = sprintf('0%d',month);
else
    month = sprintf('%d',month);
end
a_file = sprintf('%s_%s_%d.mat',day,month,year)

%implementing buttondown functionality
set(gcf,'WindowButtonDownFcn', 'graphGUIController(17)');

%saving all the sensors activated during current day
save sensor_list sensor_list; 

%saving information for zoom out
save xmin xmin
save xmax xmax
save ymin ymin
save ymax ymax
save a_file

%saving data for plot activity lines
offset = length(sensor_list)*2;
save offset
save ydays

%some buttons
uicontrol('Style','pushbutton',...
			'Position',[20 1 50 20],...
			'String','Time',...
			'CallBack','graphGUIController(13)');
        
uicontrol('Style','pushbutton',...
			'Position',[80 1 70 20],...
			'String','Interval',...
			'CallBack','graphGUIController(4)');        

uicontrol('Style','pushbutton',...
			'Position',[160 1 75 20],...
			'String','Time (Sec)',...
			'CallBack','graphGUIController(14)'); 

uicontrol('Style','pushbutton',...
			'Position',[245 1 50 20],...
			'String','Zin',...
			'CallBack','graphGUIController(15)'); 
        
uicontrol('Style','pushbutton',...
			'Position',[305 1 50 20],...
			'String','Zout',...
			'CallBack','graphGUIController(16)'); 

uicontrol('Style','pushbutton',...
			'Position',[365 1 50 20],...
			'String','+Label',...
			'CallBack','graphGUIController(18)'); 

uicontrol('Style','pushbutton',...
			'Position',[425 1 50 20],...
			'String','-Label',...
			'CallBack','graphGUIController(19)'); 

uicontrol('Style','pushbutton',...
			'Position',[485 1 60 20],...
			'String',' V Labels',...
			'CallBack','graphGUIController(20)'); 

uicontrol('Style','pushbutton',...
			'Position',[555 1 60 20],...
			'String','V Act',...
			'CallBack','graphGUIController(21)'); 
        
        
hold off;
    