function keywords(filename,maindir)

% KEYWORDS  Generates keywords for a Textpad 4 syntax highlighting file.
%    KEYWORDS('FILENAME') searches for all m-file functions in the
%    Matlab path and saves them as a list to the ascii file FILENAME.
%    Note that an existing file with the same name will be overwritten!
%
%    KEYWORDS('FILENAME','MAINDIR') searches in and below the
%    directory MAINDIR only.
%
%    The keyword list is arranged into four sections:
%    [Keywords 1]  flow control statements (if, else, etc.)
%    [Keywords 2]  common Matlab commands
%    [Keywords 3]  toolbox/blockset commands
%    [Keywords 4]  obsolete commands
%
%    After generating the keywords file FILENAME, copy the contents
%    of FILENAME into a TextPad 4 syntax file with appropriate headers
%    (see e.g. matlabR13.syn).
%    You have to manually remove undesired directories, e.g. the demos
%    and gui directories, from the generated list.

%    Jeroen Ploeg    September 18, 2002    mailto: ploeg@wt.tno.nl
%    Version 4.03


% Input checking

error(nargchk(1,2,nargin))
if nargin==2 & exist(maindir,'dir')~=7
    error('Input directory doesn''t exist!')
end


% Get cell array of subdirectories

if nargin==1
    mpath=[path pathsep];
    seplist=[0 findstr(mpath,pathsep)];
    S=cell(length(seplist)-1,1);
    for i=1:length(seplist)-1   % create cell array of Matlabpath directories
        S{i}=mpath(seplist(i)+1:seplist(i+1)-1);
    end
else
    S=tree(maindir,inf);        % get all relevant directories
end


% Sort all directories

for i=1:4                       % initialise structure array keywords
    key(i).dir=cell(0);
end
for i=1:length(S)
    if ~isempty(findstr(lower(S{i}),'obsolete'))
            key(4).dir=[key(4).dir;S(i)];   % obsolete command dirs
    else
        if isempty(findstr(lower(S{i}),lower(fullfile(matlabroot,'toolbox','matlab',''))))
            key(3).dir=[key(3).dir;S(i)];   % toolbox dirs
        else
            key(2).dir=[key(2).dir;S(i)];   % standard matlab dirs
        end
    end
end


% Get keyword lists

key(1).com={'global';'function';'try';'catch';'end';'break';'case';'elseif';'else';'for';'if';'switch';'otherwise';'while'};
key(2).com=getlist(key(2).dir,[key(1).com;{'contents';'readme'}]);
key(3).com=getlist(key(3).dir,[key(1).com;{'contents';'readme'}]);
key(4).com=getlist(key(4).dir,{'contents';'readme'});


% Print lists to text file

head={'\n[Keywords 1]\n;===== FLOW CONTROL STATEMENTS ======\n'
      '\n[Keywords 2]\n;====== COMMON MATLAB COMMANDS ======\n'
      '\n[Keywords 3]\n;========= TOOLBOX COMMANDS =========\n'
      '\n[Keywords 4]\n;========= OBSOLETE COMMANDS ========\n'};
fid=fopen(filename,'wt');
for i=1:length(head)
    fprintf(fid,head{i});
    for j=1:length(key(i).com)
        fprintf(fid,'%s\n',key(i).com{j});
        disp(key(i).com);
    end
end
st=fclose(fid);
if ~st
    disp(['Keywords succesfully written to "' filename '".'])
end


%======================================================================
% TREE
%   Get all subdirectories except private and class directories.
%   The output of this function is a cell array containing the entire
%   directory path.
%======================================================================

function S=tree(main,levels)

S={main};                                                   % initialise the tree in conception
mainlevel=length(findstr(main,filesep));                    % main folder level index
i=1;
while length(S)>=i & length(findstr(S{i},filesep))-mainlevel<levels
    subdir=dir(S{i});                                       % get the folder contents
    subdir={subdir([subdir.isdir]).name}';                  % retrieve the subfolder names
    subdir=subdir(~(strncmp('.',subdir,1) | ...             % Remove the '.' and '..' folders, ...
                    strncmp('@',subdir,1) | ...             % ... the class folders, ...
                    strcmp('private',lower(subdir)) | ...   % ... the private folders, ...
                    strcmp('sfprj',lower(subdir))));        % ... and the StateFlow folders.
    for j=1:size(subdir,1)                                  % add subfolders to the library path
        S{end+1,1}=fullfile(S{i},subdir{j},'');
    end
    i=i+1;
end
S=sort(S);


%======================================================================
% GETLIST
%       Get all m-file commands from an input list of directories,
%       except the ones mentioned in the second input list. The output
%       is a cell array of m-file commands.
%======================================================================

function list=getlist(dirlist,excludelist)

list=cell(0);
for i=1:length(dirlist)
    files=what(dirlist{i});
    list=[list;{[';----- ' dirlist{i} ' -----']};setdiff(strrep(lower(files.m),'.m',''),excludelist)];
end